/*******************************************************************************************************
 * Sprava vlakoveho hlaseni na displeji. Ve smycce se na prvnim radku displeje 16x2 zobrazuje po rade:
 * 1. Aktualni cas ziskany z hostitelskeho PC, 2. Cislo linky a cilova stanice, 3. seznam zastavek na 
 * trase, 4. Pristi zastavka. 5. Stav obslouzeni tlacitka Zastavit.
 * Lukas Fritsch
 * rev. 17.7.11.0
 *******************************************************************************************************/
 
#include "DisplayInTrain.h"
#include "StringManipulationRoutines.h"

char CDisplayInTrain::strStationsCaption[]="Pres zastavky:"; //titulek pred seznamem zastavek na trase
char CDisplayInTrain::strNextStationCaption[]="Pristi zastavka:";
char CDisplayInTrain::strStationOnRequestCaption[]="Zastavka na znameni. Pro zastaveni stisknete tlacitko signalizace.";
char CDisplayInTrain::strLeaveTrain[]="Konecna stanice. Prosime, vystupte.";
char CDisplayInTrain::strStationOnRequestAcceptedCaption[]="Zastavime.";
char CDisplayInTrain::strTrainIsStarting[]="Ukoncete vystup a nastup, vlak je pripraven k odjezdu.";
char CDisplayInTrain::strDoNotOpenDoor[]="Prosime, vyckejte do uplneho zastaveni vlaku.";


CDisplayInTrain::CDisplayInTrain()//konstruktor instance tridy CDisplayInTrain
{
  lcd=new LiquidCrystal(12, 11, 5, 4, 9, 8);//rs, enable, d4, d5, d6, d7. Vytvoreni instance LiquidCrystal
  lcd->begin(COLS_IN_ROW,2);  //dvouradkovy displej, 16 znaku/radek
  lcd->clear(); //smazani displeje
  bLineA=true;  //zacina se linkou A
  iCurrentDisplay=TIME_DISPLAY; //aktualni zobrazovana informace
  bStationOnRequestKeyPressed=false;  //tlacitko Zastavky na znameni nebylo stisknuto
  bIsInBoot=false;
}

bool CDisplayInTrain::Init()
//inicializace polozek instance
{
  Serial.begin(9600);//nastaveni serioveho kanalu
  pinMode(STATION_ON_REQUEST,INPUT);  //vstup tlacitka Zastavky na znameni
  pinMode(STATION_ON_REQUEST_ACCEPTED_LED,OUTPUT);  //indikace stisknuteho tlacitka Zastavky na znameni
  digitalWrite(STATION_ON_REQUEST_ACCEPTED_LED,LOW);  //indikace zhasnuta
  lcd->clear(); //smazani displeje
  lcd->noCursor();
  lcd->setCursor(0,0);
  lcd->print(F("Inicializuji..."));
  delay(TIME_TO_VIEW_TEXT);
  lcd->clear(); 
  if(digitalRead(STATION_ON_REQUEST)==LOW) //pri startu aplikace bylo stisknuto tlacitko
  {
    bIsInBoot=true;
    lcd->setCursor(0,0);
    lcd->print(F("Cekam na data..."));
    digitalWrite(STATION_ON_REQUEST_ACCEPTED_LED,HIGH);  //indikace rozsvicena
    WriteLinesDataToEEPROM();
    delay(TIME_TO_VIEW_TEXT);
    lcd->clear();
    bStationOnRequestKeyPressed=false;
    digitalWrite(STATION_ON_REQUEST_ACCEPTED_LED,LOW);
  }
  ReadLinesDataFromEEPROM();  //nacteni linek A, B z eeprom uP
  CreateListOfStations(strStationsLineA); //vytvoreni dat k seznamu stanic linky A
  iStationId=0; //id aktualni stanice
  if(iNrOfStations>1) //alespon 2 stanice v seznamu
  {
    iNrOfRepeatsOfThisStation=int((ListOfStations[iStationId]&0xf0000000)>>28);//pocet opakovani smycky k dane stanici
    if(GetStationName(iNrOfStations-1,strEndStation,NEXT_STATION_MAX_LEN))  //ziskani nazvu stanice
    {
      GenerateLineCaption(strEndStation); //cislo vlaku + cilova stanice
    }
    else
    {
      GenerateLineCaption("???"); //nezname informace
    }
  }
}

bool CDisplayInTrain::WriteLinesDataToEEPROM()
{
 /*
  * Cteni dat linek A, B ze serioveho kanalu hostitelskeho PC a jejich nasledne nahrani do eeprom
  */
  int iLoops;
  int iRetCode=0;
  int iAddress=EEPROM_DATA_START_ADDRESS;//adresa zapisu dat linek do eeprom
  int iChar;
  iLoops=0;
  do
  {
    iReadBytes=Serial.readBytesUntil('\n', rxBuffer,RX_BUFFER_SZ);
    if(iReadBytes==0)
      iLoops++;
  }while((iReadBytes==0)&&(iLoops<120));  //cekani na prijeti prvnich dat
  if(iReadBytes>0)  //data byla prijata
  {
    lcd->clear();
    lcd->setCursor(0,0);
    lcd->print(F("Prijimam data..."));
    /**** Cteni dat, casti A (cislo linky) ********/
    rxBuffer[iReadBytes]='\0';  //zakoncit nulou
    if(StartsWith(rxBuffer,"WDPA"))  //nasleduji data, cast A
    {
      Serial.println("WDPAA"); //potvrzeni   odesilateli
      iReadBytes=Serial.readBytesUntil('\n', rxBuffer,RX_BUFFER_SZ); 
      if(iReadBytes>0) //data byla prijata
      {
        iChar=0;  //index znaku pro zapis do eeprom
        while(iChar<iReadBytes&&iChar<TRAIN_NO_MAX_LEN) //pro vsechny znaky retezce nebo dokud neni dosazeno maximalniho poctu znaku v retezci
        {
          EEPROM.update(iAddress,rxBuffer[iChar]); //zapis znaku do eeprom. K zapisu dojde pouze v pripade, ze se jiz zapsany znak lisi od noveho znaku
          iAddress++; //dalsi adresa
          iChar++;  //dalsi znak
        }
        if(iChar<TRAIN_NO_MAX_LEN)  //nebylo dosazeno maximalni delky retezce
        {
          EEPROM.update(iAddress,0); //zapis ukoncovaciho znaku retezce (0)
          iAddress++; //dalsi adresa
        }
        Serial.println("WDPAA");  //potvrzeni
      }
      else
      {
        iRetCode=3; //nebylo nic prijato
      }
    }//end if StartsWith
    else
    {
      iRetCode=2; //prijat neocekavany ramec
    }
    /***** Cteni dat, casti B (zastavky linky A) **********/
    bool bContinue=true;
    do
    {
      iReadBytes=Serial.readBytesUntil('\n', rxBuffer,RX_BUFFER_SZ);  //cteni B do oddelovace nebo do dosazeni kapacity 
      if(iReadBytes>0)  //byl prijat alespon jeden B
      {
        rxBuffer[iReadBytes]='\0';  //zakoncit nulou
        if(StartsWith(rxBuffer,"WDPB"))  //nasleduji data, cast B
        {
          Serial.println("WDPBA"); //potvrzeni   odesilateli
          iReadBytes=Serial.readBytesUntil('\n', rxBuffer,RX_BUFFER_SZ); //cteni B do oddelovace nebo do dosazeni kapacity
          if(iReadBytes>0) //data byla prijata
          {
            iChar=0;  //index znaku pro zapis do eeprom
            while(iChar<iReadBytes&&iChar<STATIONS_MAX_LEN) //pro vsechny znaky retezce nebo dokud neni dosazeno maximalniho poctu znaku v retezci
            {
              EEPROM.update(iAddress,rxBuffer[iChar]); //zapis znaku do eeprom. K zapisu dojde pouze v pripade, ze se jiz zapsany znak lisi od noveho znaku
              iAddress++; //dalsi adresa
              iChar++;  //dalsi znak
            }
          }
          if(iReadBytes<=0)
          {
            iRetCode=3; //nebylo nic prijato
          }
          else
          {
            Serial.println("WDPBA");  //potvrzeni prijeti odesilateli
          }
        }//end if StartsWith
        else
        {
          bContinue=false;
        }
      }//end if iReadBytes>0
      else
        iRetCode=3; //nebylo nic prijato
    }while(bContinue&&(iRetCode==0));
    EEPROM.update(iAddress,0); //zapis ukoncovaciho znaku retezce (0)
    iAddress++; //dalsi adresa
    /***** Cteni dat, casti C (cislo vlaku linky B) **********/
    //rxBuffer naplnen v predchozim bloku
    if(StartsWith(rxBuffer,"WDPC"))  //nasleduji data, cast A
    {
      Serial.println("WDPCA"); //potvrzeni   odesilateli
      iReadBytes=Serial.readBytesUntil('\n', rxBuffer,RX_BUFFER_SZ); 
      if(iReadBytes>0) //data byla prijata
      {
        iChar=0;  //index znaku pro zapis do eeprom
        while(iChar<iReadBytes&&iChar<TRAIN_NO_MAX_LEN) //pro vsechny znaky retezce nebo dokud neni dosazeno maximalniho poctu znaku v retezci
        {
          EEPROM.update(iAddress,rxBuffer[iChar]); //zapis znaku do eeprom. K zapisu dojde pouze v pripade, ze se jiz zapsany znak lisi od noveho znaku
          iAddress++; //dalsi adresa
          iChar++;  //dalsi znak
        }
        if(iChar<TRAIN_NO_MAX_LEN)  //nebylo dosazeno maximalni delky retezce
        {
          EEPROM.update(iAddress,0); //zapis ukoncovaciho znaku retezce (0)
          iAddress++; //dalsi adresa
        }
        Serial.println("WDPCA");  //potvrzeni
      }
      else
      {
        iRetCode=3; //nebylo nic prijato
      }
    }//end if StartsWith
    else
    {
      iRetCode=2; //prijat neocekavany ramec
    }
      /***** Cteni dat, casti D (zastavky linky B) **********/
    bContinue=true;
    do
    {
      iReadBytes=Serial.readBytesUntil('\n', rxBuffer,RX_BUFFER_SZ);  //cteni B do oddelovace nebo do dosazeni kapacity 
      if(iReadBytes>0)  //byl prijat alespon jeden B
      {
        rxBuffer[iReadBytes]='\0';  //zakoncit nulou
        if(StartsWith(rxBuffer,"WDPD"))  //nasleduji data, cast B
        {
          Serial.println("WDPDA"); //potvrzeni   odesilateli
          iReadBytes=Serial.readBytesUntil('\n', rxBuffer,RX_BUFFER_SZ); //cteni B do oddelovace nebo do dosazeni kapacity
          if(iReadBytes>0) //data byla prijata
          {
            iChar=0;  //index znaku pro zapis do eeprom
            while(iChar<iReadBytes&&iChar<STATIONS_MAX_LEN) //pro vsechny znaky retezce nebo dokud neni dosazeno maximalniho poctu znaku v retezci
            {
              EEPROM.update(iAddress,rxBuffer[iChar]); //zapis znaku do eeprom. K zapisu dojde pouze v pripade, ze se jiz zapsany znak lisi od noveho znaku
              iAddress++; //dalsi adresa
              iChar++;  //dalsi znak
            }
          }
          if(iReadBytes<=0)
          {
            iRetCode=3; //nebylo nic prijato
          }
          else
          {
            Serial.println("WDPDA");  //potvrzeni prijeti odesilateli
          }
        }//end if StartsWith
        else
        {
          bContinue=false;//vsechny 4 sekce byly prijaty, ukoncit operaci zapisu
        }
      }//end if iReadBytes>0
      else
        iRetCode=3; //nebylo nic prijato
    }while(bContinue&&(iRetCode==0));
    EEPROM.update(iAddress,0); //zapis ukoncovaciho znaku retezce (0)
  }//end if readBytes>0
  else
  {
    iRetCode=1;//nebyl zahajen prenos dat do timeoutu
  }
  if(iRetCode==0)
  {
    EEPROM.update(EEPROM_DATA_VALID_ADDRESS,EEPROM_DATA_VALID_FLAG); //zapis priznaku platnosti eeprom dat
    lcd->clear();
    lcd->setCursor(0,0);
    lcd->print(F("Data prijata."));
  }
  else
  {
    if(iRetCode!=1)//ponechat puvodni data
    {
      EEPROM.write(EEPROM_DATA_VALID_ADDRESS,~EEPROM_DATA_VALID_FLAG); //zapis priznaku platnosti eeprom dat
      lcd->clear();
      lcd->setCursor(0,0);
      lcd->print(F("Chyba: "));
      lcd->setCursor(8,0);
      lcd->print(iRetCode);
    }
  }
  return iRetCode==0;
}

void CDisplayInTrain::ReadLinesDataFromEEPROM()
/*
 * Cteni dat linek A, B z EEPROM procesoru
 */
{
  char cChar;
  int iAddress=EEPROM_DATA_VALID_ADDRESS;
  int iChar=0;
  /**** Cteni priznaku platnosti dat linek ************/
  bEEPROMDataValid=(EEPROM.read(iAddress)==EEPROM_DATA_VALID_FLAG);
  if(bEEPROMDataValid)
  {
    iAddress=EEPROM_DATA_START_ADDRESS;
    /**** Cteni dat linky A ****************/
    do
    {
      cChar=EEPROM.read(iAddress);  //cteni B z eeprom
      iAddress++; //dalsi adresa
      strTrainNoLineA[iChar]=cChar;
      iChar++;  //dalsi znak
    }while(cChar!='\0'&&iChar<TRAIN_NO_MAX_LEN);  //dokud neni precten ukoncovaci znak retezce nebo dokud nebyla prekrocena delka retezce
    if(iChar>=TRAIN_NO_MAX_LEN)//prekrocena maximalni delka retezce, explicitni zapis ukoncovaciho znaku retezce
    {
      strTrainNoLineA[TRAIN_NO_MAX_LEN]='\0';
    }
    iChar=0;
    do
    {
      cChar=EEPROM.read(iAddress);
      iAddress++;
      strStationsLineA[iChar]=cChar;
      iChar++;
    }while(cChar!='\0'&&iChar<STATIONS_MAX_LEN);
    if(iChar>=STATIONS_MAX_LEN)
    {
      strStationsLineA[STATIONS_MAX_LEN]='\0';
    }
    /**** Cteni dat linky B ****************/
    iChar=0;
    do
    {
      cChar=EEPROM.read(iAddress);  //cteni B z eeprom
      iAddress++; //dalsi adresa
      strTrainNoLineB[iChar]=cChar;
      iChar++;  //dalsi znak
    }while(cChar!='\0'&&iChar<TRAIN_NO_MAX_LEN);  //dokud neni precten ukoncovaci znak retezce nebo dokud nebyla prekrocena delka retezce
    if(iChar>=TRAIN_NO_MAX_LEN)//prekrocena maximalni delka retezce, explicitni zapis ukoncovaciho znaku retezce
    {
      strTrainNoLineB[TRAIN_NO_MAX_LEN]='\0';
    }
    iChar=0;
    do
    {
      cChar=EEPROM.read(iAddress);
      iAddress++;
      strStationsLineB[iChar]=cChar;
      iChar++;
    }while(cChar!='\0'&&iChar<STATIONS_MAX_LEN);
    if(iChar>=STATIONS_MAX_LEN)
    {
      strStationsLineB[STATIONS_MAX_LEN]='\0';
    }
  }//eeprom data jsou platna
  else
  { //eeprom data nejsou platna, nastavit nulovou delku prislusnym retezcum
    strTrainNoLineA[0]='\0';
    strTrainNoLineB[0]='\0';
    strStationsLineA[0]='\0';
    strStationsLineB[0]='\0';
  }
}

bool CDisplayInTrain::GetLocalTime()
//Ziskani aktualniho casu z hostitelskeho pc. Vrati true = OK, false = NG
{
  Serial.println("GLT?");
  iReadBytes=Serial.readBytesUntil('\n', rxBuffer,RX_BUFFER_SZ);
  if(iReadBytes>=GLT_RESPONSE_MIN_LEN)
  {
    rxBuffer[iReadBytes]='\0';//vytvoreni nulou ukonceneho retezce
    if(rxBuffer[0]=='G'&&rxBuffer[1]=='L'&&rxBuffer[2]=='T'&&rxBuffer[3]==' ')  //prefix ok
    {
      strTime=&rxBuffer[4]; //pointer na aktualni cas
      return true;
    }
  }
  return false;
}

void CDisplayInTrain::ScrollInText(char * _str, int _iStrLen, int _iRow)
/* 
 *  Roluje text, na nejz ukazuje vstupni parametr _str, pres displej zprava doleva.
 */
{
  int iCounter=0;
  int iChar=0; 
  int iStartFromChar=0; //poradi znaku v retezci, od nehoz se maji vypisovat znaky v radku
  int iCol=COLS_IN_ROW; //cislo sloupce v radku, od nehoz jsou vypisovany znaky retezce zleva doprava
  int iCharInRow; //poradi znaku v radku displeje
  while(iChar<=_iStrLen)
  {
    iCounter=iCol;
    iCharInRow=0;
    while(iCharInRow<(iCol-1))//zarovnani retezce zleva mezerami
    {
      strDispOneRow[iCharInRow]=' ';
      iCharInRow++;
    }
    while(iCounter<=COLS_IN_ROW)//pro vsechny aktivni sloupce v radku 
    {
      if(iChar<_iStrLen)
      {
        if(_str[iChar]=='#'||(iChar>0&&_str[iChar-1]=='$'&&iCounter==0))
          iChar++;
        else
        {
         if(_str[iChar]=='$')
           iChar+=2;
        }
        if(iChar<_iStrLen)
        { 
          strDispOneRow[iCharInRow]=_str[iChar];
        }
        else
        {
           strDispOneRow[iCharInRow]=' '; //pravostranne zarovnani mezerami
        }
      }
      else
      {
          strDispOneRow[iCharInRow]=' ';//pravostranne zarovnani mezerami
      }
      iCounter++;
      iChar++;
      if(iCharInRow<(COLS_IN_ROW-1))
        iCharInRow++;
    }
    lcd->setCursor(0,_iRow);
    strDispOneRow[++iCharInRow]='\0';
    lcd->print(strDispOneRow);
    if(iCol>1)  //jeste se nezacina od prvniho sloupce
      iCol--;
    else
    {//vypis znaku na displeji zacina od prvniho sloupce
        if(_str[iStartFromChar]=='$')//prvni znak v radku byl ridici, preskocit ridici znak a jeho parametr
        {
           iStartFromChar+=2;
        }
        else
        {
          if(_str[iStartFromChar]=='#') //prvni znak v radku byl ridici, preskocit ridici znak
          {
            iStartFromChar++;
          }
        }
        iStartFromChar++; //byl-li prvni znak jakymkoliv ridicim znakem, na jeho miste se vypsal nasledujici znak, takze v dalsi smycce se uz nemusi vypisovat -> jeste jedna inkrementace
    }
    iChar=iStartFromChar;  
    delay(SCROLL_SPEED);
  }//end while iChar<_iStrLen

}

int CDisplayInTrain::CreateListOfStations(char* _strAll)
/*
 * Vytvoreni seznamu zastavek na dane lince. Na seznam ukazuje vstupni parametr _strAll. Vrati pocet zastavek na lince.
 */
{
  int iCounter=0;
  int iLen=GetStrLen(_strAll);
  int iStationId=0;
  do
  {
    ListOfStations[iStationId]=0;
    while((iCounter<iLen)&&(_strAll[iCounter]==' '))  //preskoceni white spaces
    {
      iCounter++;
    }
    if(iCounter<iLen) //neni konec retezce
    {
      if(_strAll[iCounter]=='#')//ridici znak zastavky na znameni
      {
        if((iCounter+1)<iLen)
        {
          ListOfStations[iStationId]=iCounter+1;
          ListOfStations[iStationId]|=(STATION_TYPE_ON_REQUEST);//nastaveni priznaku zastavky na znameni
        }
      }
      else
      {
        ListOfStations[iStationId]=iCounter;
        ListOfStations[iStationId]|=(STATION_TYPE_DEFAULT);//defaultni priznak
      }
    }
    do
    {
      iCounter++;
    }while((iCounter<iLen)&&(_strAll[iCounter]!='$')&&(_strAll[iCounter]!='-'));  //nalezeni suffixu s poctem opakovani pripadne konce stanice
    int iStart=ListOfStations[iStationId];
    ListOfStations[iStationId]|=(long((iCounter-iStart)&0xff)<<16);
    if(_strAll[iCounter]=='$')  //suffix s poctem opakovani
    {
      iCounter++;
      if(iCounter<iLen)
      {
        if(isDigit(_strAll[iCounter]))  //jde o cislici
        {
          ListOfStations[iStationId]|=(long(_strAll[iCounter]-'0')<<28); //pocet opakovani do hornich 4 bitu 4B slova
        }
      }
    }
    while((iCounter<iLen)&&(_strAll[iCounter]!='-'))  //nalezeni konce stanice
    {
      iCounter++;
    }
    iCounter++;
    iStationId++;
  }while((iCounter<iLen)&&(iStationId<MAX_STATIONS));
  iNrOfStations=iStationId;
  while(iStationId<MAX_STATIONS)//ostatni stanice neobsazeny
  {
    ListOfStations[iStationId]=0;
    ListOfStations[iStationId]|=(STATION_TYPE_NONE);
    iStationId++;
  }
  return iNrOfStations;
}

bool CDisplayInTrain::GetStationName(int _iStationId,char* strDest,int iStrDestMaxLen)
//Ziskani nazvu stanice podle jejiho Id (vstupni parametr _iStationId). Nazev je zkopirovan na pametove misto
//urcene pointerem strDest, jehoz maximalni velikost je iStrDestMaxLen bytu. Vrati true = OK, false = NG.
{
   int iChar=0;
   if(_iStationId>=0&&_iStationId<MAX_STATIONS)
   {
     int iStartAt=ListOfStations[_iStationId]&0xffff; //index prvniho znaku nazvu stanice
     int iStrLen=((ListOfStations[_iStationId]&0x00ff0000)>>16);  //delka nazvu stanice ve 3. bajtu 4B slova
     char* strPtr;
     if(bLineA)
      strPtr=strStationsLineA;
     else
      strPtr=strStationsLineB;
     while(strPtr[iStartAt]=='#')//preskoceni ridiciho znaku
      iStartAt++;
     while(iChar<iStrLen&&iChar<iStrDestMaxLen) //kopie znaku
     {
        strDest[iChar]=strPtr[iStartAt];
        iChar++;
        iStartAt++;
     }
     strDest[iChar]='\0';//nulou ukonceny retezec
   }
   return (iChar>0);            
}

void CDisplayInTrain::GenerateLineCaption(char* _strEndStation)
// Ziskani identifikatoru vlaku (cislo vlaku + cilova stanice (_strEndStation).
{
  int iChar=0;
  int iLen;
  char* pstr;
  if(bLineA)
  {
    iLen=GetStrLen(strTrainNoLineA);
    pstr=strTrainNoLineA;
  }
  else
  {
    iLen=GetStrLen(strTrainNoLineB);
    pstr=strTrainNoLineB;
  }
  if(iLen<=(LINE_CAPTION_MAX_LEN-NEXT_STATION_MAX_LEN)&&iLen>0)
  {
    while(iChar<iLen)
    {
      strLine[iChar]=pstr[iChar];
      iChar++;
    }
  }
  else
  {
    strLine[0]='U';
    strLine[1]='5';
    strLine[2]='2';
    iLen=3;
  }
  strLine[iLen]=' ';
  iLen++;
  strLine[iLen]='-';
  iLen++;
  strLine[iLen]='>';
  iLen++;
  iChar=0;
  while(_strEndStation[iChar]!='\0'&&iChar<NEXT_STATION_MAX_LEN)
  {
    strLine[iChar+iLen]=_strEndStation[iChar];
    iChar++;
  }
  strLine[iChar+iLen]='\0';
}

void CDisplayInTrain::OnLoop()
// obsluha udalosti loop
{

       switch(iCurrentDisplay)//aktualni informace k zobrazeni
      {
         case TIME_DISPLAY://aktualni cas
         {
            if(GetLocalTime())
            {
              iStrLen=GetStrLen(strTime);
              iInitialCol=(COLS_IN_ROW-iStrLen)/2;
              lcd->setCursor(iInitialCol,0);
              lcd->print(strTime);
            }
            else
            {
              iInitialCol=(COLS_IN_ROW-5)/2;
              lcd->setCursor(iInitialCol,0);
              lcd->print("02:27");
            }
            delay(TIME_TO_VIEW_TEXT);
            lcd->clear();
            if(iNrOfStations>1)
              iCurrentDisplay++;
            break;  
         }
         case LINE_DISPLAY://identifikator vlaku
         {
            iStrLen=GetStrLen(strLine);
            if(iStrLen<=COLS_IN_ROW)
            {
              iInitialCol=(COLS_IN_ROW-iStrLen)/2;
              lcd->setCursor(iInitialCol,0);
              lcd->print(strLine);
              delay(TIME_TO_VIEW_TEXT);
              lcd->clear();
            }
            else
            {
              ScrollInText(strLine,iStrLen,0);
            }
            iCurrentDisplay++;
            break;
         }
         case STATIONS_DISPLAY://seznam zastavek
         {
            iStrLen=GetStrLen(strStationsCaption);
            if(iStrLen<=COLS_IN_ROW)
            {
              iInitialCol=(COLS_IN_ROW-iStrLen)/2;
              lcd->setCursor(iInitialCol,0);
              lcd->print(strStationsCaption);
              delay(TIME_TO_VIEW_TEXT);
            }
            else
            {
              ScrollInText(strStationsCaption,iStrLen,0);
            }
            if(bLineA)
            {
              iStrLen=GetStrLen(strStationsLineA);
              if(iStrLen<=COLS_IN_ROW)
              {
                iInitialCol=(COLS_IN_ROW-iStrLen)/2;
                lcd->setCursor(iInitialCol,1);
                lcd->print(strStationsLineA);
                delay(TIME_TO_VIEW_TEXT);
              }
              else
              {
                ScrollInText(strStationsLineA,iStrLen,1);
              }
            }
            else
            {
              iStrLen=GetStrLen(strStationsLineB);
              if(iStrLen<=COLS_IN_ROW)
              {
                iInitialCol=(COLS_IN_ROW-iStrLen)/2;
                lcd->setCursor(iInitialCol,1);
                lcd->print(strStationsLineB);
                delay(TIME_TO_VIEW_TEXT);
              }
              else
              {
                ScrollInText(strStationsLineB,iStrLen,1);
              }
            }
            lcd->clear();
            iCurrentDisplay++;
            break;
         }
         case NEXT_STATION_DISPLAY://pristi zastavka
         {
            if(((ListOfStations[iStationId]&&0x0f000000))!=STATION_TYPE_NONE)
            {
              Serial.print("PS ");
              Serial.println(F("NextStation"));
              iStrLen=GetStrLen(strNextStationCaption);
              iInitialCol=(COLS_IN_ROW-iStrLen)/2;
              lcd->setCursor(iInitialCol,0);
              delay(TIME_TO_VIEW_TEXT);
              lcd->print(strNextStationCaption);
              if(GetStationName(iStationId,strNextStation,NEXT_STATION_MAX_LEN))
              {
                Serial.print("PS ");
                Serial.println(strNextStation);
                iStrLen=GetStrLen(strNextStation);
                delay(TIME_BEFORE_PRINT);
                ScrollInText(strNextStation,iStrLen,1);
              }
              else
              {
                iInitialCol=(COLS_IN_ROW-4)/2;
                lcd->setCursor(iInitialCol,0);
                lcd->print("????");
              }
              delay(TIME_TO_VIEW_TEXT);
              lcd->clear();
              if(StationOnRequest())//jde o zastavku na znameni
              {
                if(!RequestAccepted()) //tlacitko nebylo stisknuto
                {
                  Serial.print("PS ");
                  Serial.println(F("PushButton"));
                  iStrLen=GetStrLen(strStationOnRequestCaption);
                  if(iStrLen<=COLS_IN_ROW)
                  {
                    iInitialCol=(COLS_IN_ROW-iStrLen)/2;
                    lcd->setCursor(iInitialCol,0);
                    lcd->print(strStationOnRequestCaption);
                    delay(TIME_TO_VIEW_TEXT);
                    lcd->clear();
                  }
                  else
                  {
                    ScrollInText(strStationOnRequestCaption,iStrLen,0);
                  }
                  delay(3000);
                }
              } 
            }
            iCurrentDisplay++;
            break;
         }
         case STOP_ON_REQUEST_ACCEPTED://tlacitko zastaveni
         {
            if(bStationOnRequestKeyPressed) //tlacitko Zastavit bylo stisknuto
            {
              if(((ListOfStations[iStationId]&&0x0f000000))!=STATION_TYPE_NONE)
              {
                if(StationOnRequest())//jde o zastavku na znameni
                {
                  Serial.print("PS ");
                  Serial.println(F("ButtonPushed"));
                  iStrLen=GetStrLen(strStationOnRequestAcceptedCaption);
                  if(iStrLen<=COLS_IN_ROW)
                  {
                    iInitialCol=(COLS_IN_ROW-iStrLen)/2;
                    lcd->setCursor(iInitialCol,0);
                    lcd->print(strStationOnRequestAcceptedCaption);
                    delay(TIME_TO_VIEW_TEXT);
                    lcd->clear();
                  }
                  else
                  {
                    ScrollInText(strStationOnRequestAcceptedCaption,iStrLen,0);
                  }
                } 
              }
            }
            iCurrentDisplay++;
            break;
         }
         case MANIPULATION_PURPOSES_DISPLAY://vycisleni udaju pro dalsi smycku
         {
            if(iNrOfRepeatsOfThisStation<1) //pocet opakovani dane stanice vycerpan
            {   
              digitalWrite(STATION_ON_REQUEST_ACCEPTED_LED,LOW);
              bIsNotCurrentStationOnRequest=!StationOnRequest();
              if(iStationId<(iNrOfStations-1))  //nedosazeno cilove stanice
              {
                
                iStationId++;
              }
              else//dosazeno cilove stanice
              {
                  Serial.print("PS ");
                  Serial.println(F("WaitToStop"));
                  iStrLen=GetStrLen(strDoNotOpenDoor);
                  if(iStrLen<=COLS_IN_ROW)
                  {
                    iInitialCol=(COLS_IN_ROW-iStrLen)/2;
                    lcd->setCursor(iInitialCol,0);
                    lcd->print(strDoNotOpenDoor);
                    delay(TIME_TO_VIEW_TEXT);
                    lcd->clear();
                  }
                  else
                  {
                    ScrollInText(strDoNotOpenDoor,iStrLen,0);
                  }
                  delay(TIME_TO_VIEW_TEXT);
                  Serial.print("PS ");
                  Serial.println(F("LeaveTrain"));
                  iStrLen=GetStrLen(strLeaveTrain);
                  if(iStrLen<=COLS_IN_ROW)
                  {
                    iInitialCol=(COLS_IN_ROW-iStrLen)/2;
                    lcd->setCursor(iInitialCol,0);
                    lcd->print(strLeaveTrain);
                    delay(TIME_TO_VIEW_TEXT);
                    lcd->clear();
                  }
                  else
                  {
                    ScrollInText(strLeaveTrain,iStrLen,0);
                  }
                  bLineA=!bLineA; //negace smeru
                  if(bLineA)
                  {
                    CreateListOfStations(strStationsLineA);
                  }
                  else
                  {
                    CreateListOfStations(strStationsLineB);
                  }
                  iStationId=0;
                  if(iNrOfStations>1)
                  {
                    if(GetStationName(iNrOfStations-1,strEndStation,NEXT_STATION_MAX_LEN))
                    {
                      GenerateLineCaption(strEndStation);
                    }
                    else
                    {
                      GenerateLineCaption("???");
                    }
                    iNrOfRepeatsOfThisStation=int((ListOfStations[iStationId]&0xf0000000)>>28); //pocet opakovani nasledujici stanice
                  }
                  else
                    iNrOfRepeatsOfThisStation=0;
                }
                if(bStationOnRequestKeyPressed||bIsNotCurrentStationOnRequest)
                {
                  if(iStationId!=0)//nejde o vychozi stanici
                  {
                      Serial.print("PS ");
                      Serial.println(F("WaitToStop"));
                      iStrLen=GetStrLen(strDoNotOpenDoor);
                      if(iStrLen<=COLS_IN_ROW)
                      {
                        iInitialCol=(COLS_IN_ROW-iStrLen)/2;
                        lcd->setCursor(iInitialCol,0);
                        lcd->print(strDoNotOpenDoor);
                        delay(TIME_TO_VIEW_TEXT);
                        lcd->clear();
                      }
                      else
                      {
                        ScrollInText(strDoNotOpenDoor,iStrLen,0);
                      }
                  }
                  if(GetLocalTime())
                  {
                    iStrLen=GetStrLen(strTime);
                    iInitialCol=(COLS_IN_ROW-iStrLen)/2;
                    lcd->setCursor(iInitialCol,0);
                    lcd->print(strTime);
                  }
                  else
                  {
                    iInitialCol=(COLS_IN_ROW-5)/2;
                    lcd->setCursor(iInitialCol,0);
                    lcd->print("02:27");
                  }
                  delay(TIME_TO_WAIT_IN_STATION);
                  lcd->clear();
                  Serial.print("PS ");
                  Serial.println(F("TrainIsStarting"));
                  iStrLen=GetStrLen(strTrainIsStarting);
                  if(iStrLen<=COLS_IN_ROW)
                  {
                    iInitialCol=(COLS_IN_ROW-iStrLen)/2;
                    lcd->setCursor(iInitialCol,0);
                    lcd->print(strTrainIsStarting);
                    delay(TIME_TO_VIEW_TEXT);
                    lcd->clear();
                  }
                  else
                  {
                    ScrollInText(strTrainIsStarting,iStrLen,0);
                  }
                }
                bStationOnRequestKeyPressed=false; //shodit priznak stisku tlacitka pro dalsi zastavku
            }
            else
            {
              iNrOfRepeatsOfThisStation--;
            }
            iCurrentDisplay++;
            break;
         }//end case MANIPULATION_PURPOSES_DISPLAY
      }//end switch
      if(iCurrentDisplay>=MAX_DISPLAYS)
        iCurrentDisplay=TIME_DISPLAY;
}

bool CDisplayInTrain::RequestAccepted()
//vrati true, pokud byl detekovan stisk tlacitka Zastavit
{
  return bStationOnRequestKeyPressed;
}

void CDisplayInTrain::AcceptRequest()
// obsluha stisknuteho tlacitka Zastavit
{
  if(StationOnRequest())
  {
    bStationOnRequestKeyPressed=true;
    digitalWrite(STATION_ON_REQUEST_ACCEPTED_LED,HIGH);
  }
}

bool CDisplayInTrain::StationOnRequest()
//vrati true, pokud jde o zastavku na znameni
{
  if(iStationId>=0&&iStationId<MAX_STATIONS)
    return (ListOfStations[iStationId]&STATION_TYPE_ON_REQUEST);
  else
    return false;
}

